*DECK FLUINR
      SUBROUTINE FLUINR(FMODUL,SMODUL,IPSYS ,IPMACR,IPTRK ,IPBTF ,
     >                  IPRINT,ITPIJ ,KDIR  ,NGROUP,NBMIX ,NREGIO,
     >                  NUNKNO,MAXINR,EPSTHR,IFRITR,IACITR,IREBAL,
     >                  LEAKSW,ILEAK ,TITLE ,MATCOD,VOLUME,KEYFLX,
     >                  FUNKNO,SUNKNO,MXNL  ,ISADJ ,
     >                  DB2   ,SIGT0 ,SIGS0 ,XSCAT ,IJJ   ,NJJ   ,
     >                  IPOS  ,IFGUPS,IGCONV,DELFL ,SOURC ,PNL   )
C
C---------------------------  FLUINR  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : FLUINR
C       USE      : COMPUTE GROUP SOURCE AND CALL FLUX SOLUTION MODULE
C                  IN FAST AND RESONANCE GROUPS. FOR THERMAL GROUP,
C                  THIS ROUTINE CONTROLS THE ITERATION PROCESS,
C                  PERFORMS REBALANCING AND COMPUTES A VARIATIONAL
C                  OVER-RELAXATION PARAMETER
C
C       MODIFIED : DEAL WITH ADJOINT FLUX
C                  2000/02/01 (G. MARLEAU)
C                  2000/01/20 (T. COURAU)
C                  1994/10/27 (A. HEBERT)
C                  1994/07/21 (I. PETROVIC, G. MARLEAU)
C                  1991/04/02 (R. ROY)
C                  1991/03/15 (G. MARLEAU)
C                  1991/01/28 (A. HEBERT)
C                  1986/05/14 (G. MARLEAU)
C
C   2- ROUTINE PARAMETERS:
C    INPUT:
C      FMODUL  : FLUX SOLUTION MODULE.                EXT ROUTINE
C      SMODUL  : RESIDUAL EVALUATION MODULE.          EXT ROUTINE
C      IPSYS   : POINTER TO THE PIJ                   I
C      IPMACR  : POINTER TO THE MACROLIB              I
C      IPTRK   : POINTER TO THE TRACKING              I
C      IPBTF   : POINTER TO THE BINARY TRACKING FILE  I
C      ITPIJ   : TYPE OF CP AVAILABLE                 I
C                = 1 SCATT MOD PIJ (WIJ)
C                = 2 STAND. PIJ
C                = 3 SCATT MOD PIJ+PIJK (WIJ,WIJK)
C                = 4 STAND. PIJ+PIJK
C      KDIR    : DIRECTION FOR PC                     I
C                = 0 FOR PIJ OR WIJ
C                = K FOR PIJK OR WIJK K=1,2,3 B1
C                = K+3 FOR PIJK OR WIJK K=4,5,6 B0
C      NGROUP  : NUMBER OF GROUPS.                    I
C      NBMIX   : NUMBER OF MIXTURES.                  I
C      NREGIO  : NUMBER OF REGIONS.                   I
C      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.     I
C      MAXINR  : MAX.# PASS IN CURRENT ITERATION.     I
C      EPSTHR  : CONV. CRIT. FOR CURRENT ITERATION.   R
C      IFRITR  : NB. SUCCESSIVE FREE ITERATIONS.      I
C      IACITR  : NB. SUCCESSIVE ACCEL. ITERATIONS.    I
C      IREBAL  : REBALANCING THERMAL GROUP:           I
C              : = 0 NO REBALANCING
C              : = 1 REBALANCE NON CONVERGED GROUPS
C              : = 2 REBALANCE ALL THERMAL GROUPS
C      LEAKSW  : LEAKAGE SWITCH.                      L
C      ILEAK   : METHOD USED TO INCLUDE DB2 EFFECT    I
C                = 0 NO DB2 EFFECT
C                = 1 THE SCATTERING MODIFIED CP MATRIX
C                    IS MULTIPLIED BY A PNL FACTOR
C                = 2 THE REDUCED CP MATRIX IS MULTIPLIED BY
C                    PNL FACTOR
C                = 3 SIGS0-DB2 APPROXIMATION
C                = 4 ALBEDO APPROXIMATION
C                = 5,6,7,8,9,10 HETEROGENEOUS WITH PIJK
C                  AND FIXED B->5 OR SEARCH
C                  X->6 Y->7 Z->8C R->9 ALL->10
C      TITLE   : TITLE OF THE DRAGON CASE.            C*72
C      MATCOD  : MATERIAL CODE IN REGIONS.            I(NREGIO)
C      VOLUME  : VOLUME OF REGIONS. DIMENSION         R(NREGIO)
C      KEYFLX  : FLUX ELEMENTS IN UNKNOWN SYSTEM.     I(NREGIO)
C      FUNKNO  : UNKNOWN VECTOR SOLVED FOR.           R(NUNKNO,NGROUP)
C      SUNKNO  : SOURCE FOR SYSTEM OF UNKNOWN.        R(NUNKNO,NGROUP)
C      MXNL    : MAXIMUM ANISOTROPY                   I
C      ISADJ   : ADJOINT CALCULATION AND SAVE OPTION  I
C                = 0 ONLY DIRECT FLUX
C                = 1 DIRECT AND PSEUDO ADJOINT FLUX
C                = 2 DIRECT, PSEUDO-ADJOINT AND ADJOINT
C                = 3 DIRECT, PSEUDO ADJOINT AND
C                    GENERALIZED PSEUDO-ADJOINT
C                = 4 DIRECT, PSEUDO ADJOINT, ADJOINT,
C                    GENERALIZED PSEUDO-ADJOINT AND
C                    GENERALIZED ADJOINT
C      DB2     : LEAKAGE PARAMETERS.                  R(NGROUP)
C                FOR ITPIJ=1,2 -> DB2(G)=D(G)*B**2
C                FOR ITPIJ=3,4 -> DB2(G)=LAMBDA(B/SIGMA(G))
C    OUTPUT:
C      FUNKNO  : UNKNOWN VECTOR SOLVED FOR.           R(NUNKNO,NGROUP)
C    WORK STORAGE:
C      SIGT0   : TOTAL CROSS SECTION VECTOR           R(0:NBMIX)
C                SIGT0(0)=0.0 IS FOR VOID
C      SIGS0   : WITHIN-GROUP SCATTERING              R(0:NBMIX)
C                SIGS0(0)=0.0 IS FOR VOID
C      XSCAT   : SCATTERING VECTOR                    R(0:NBMIX*NGROUP)
C                XSCAT(0) = 0.0
C      IJJ     : LAST SCATTERING GROUP                I(0:NBMIX)
C                IJJ(0) = 0
C      NJJ     : NUMBER OF SCATTERING GROUP           I(0:NBMIX)
C                NJJ(0) = -NGROUP
C      IPOS    : POSITION SELF SCATTERING IN XSCAT    I(0:NBMIX)
C                IPOS(0)= NGROUP+1
C      IFGUPS  : FIRST GROUP WITH UPSCATTERING        I(MXNL)
C      IGCONV  : GROUP CONVERGENCE IDENTIFIER         I(NGROUP)
C      DELFL   : ERROR ON FLUXES                      R(NUNKNO,NGROUP)
C      SOURC   : ONE GROUP TOTAL SOURCES              R(NUNKNO)
C      PNL     : NON LEAKAGE PROBABILITY              R(NGROUP)
C
C---------------------------   FLUINR  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,NAMSBR='FLUINR')
C----
C  ROUTINE PARAMETERS
C----
      EXTERNAL    FMODUL,SMODUL
      INTEGER     IPSYS,IPMACR,IPTRK,IPBTF,IPRINT,ITPIJ,KDIR,
     >            NGROUP,NBMIX,NREGIO,NUNKNO,MAXINR,IFRITR,
     >            IACITR,IREBAL,ILEAK,MXNL,ISADJ
      CHARACTER   TITLE*72
      INTEGER     MATCOD(NREGIO),KEYFLX(NREGIO),
     >            IJJ(0:NBMIX),NJJ(0:NBMIX),IPOS(0:NBMIX),
     >            IFGUPS(MXNL),IGCONV(NGROUP)
      LOGICAL     LEAKSW
      REAL        EPSTHR,VOLUME(NREGIO),FUNKNO(NUNKNO,NGROUP),
     >            SUNKNO(NUNKNO,NGROUP),DB2(NGROUP),
     >            SIGT0(0:NBMIX),SIGS0(0:NBMIX),XSCAT(0:NBMIX*NGROUP),
     >            DELFL(NUNKNO,NGROUP),SOURC(NUNKNO),
     >            PNL(NGROUP)
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     IGAMMA,IRESTE,JREBAL,JPIS
C----
C  FUNCTIONS
C----
      REAL        FLUFUI
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   CGRPNM*12,CAN*3,CNSIGW*12
      INTEGER     LONS0,IDIR,IDIAGA,ICNL,KRSK,IDIREC,LMAXIN,IGR,
     >            INEXTG,ILASTG,ITHERI,IGMAXD,IFRSTG,IALLCV,
     >            ILCMLN,ILCMTY,IELEM,IBM,IGSCAT,IRLOC,IREG,
     >            ILSG,JGR,NTOTGS,IFGP,ILGP,ITHRCV
      REAL        WOVER,TRELER,FACT,DB2NEW,GDPHI,GMPHI,RELCNV
      LOGICAL     LFGTHR,LACVAR
      DOUBLE PRECISION ZL2
C----
C  DATA
C----
      CHARACTER   CNFDIR(0:3)*5
      SAVE        CNFDIR
      DATA        CNFDIR
     >           /'FLUX','CUR-X','CUR-Y','CUR-Z'/
C----
C  FOR DIRECTIONS KDIR=0 USE ISOTROPIC SCATTERING
C  FOR DIRECTIONS KDIR=1,2,3 USE ANISOTROPIC SCATTERING B1
C  FOR DIRECTIONS KDIR=4,5,6 USE ANISOTROPIC SCATTERING B0
C  KRSK = LOCATION OF FIRST ELEMENT FOR VARIOUS DIRECTION
C----
      IF(ISADJ .EQ. 0) THEN
        CAN='D 0'
      ELSE
        CAN='A 0'
      ENDIF
      IF(KDIR .EQ. 0) THEN
        CNSIGW='DRAGON-S0XSC'
        LONS0=NBMIX+1
        IDIR=0
        IDIAGA=0
        ICNL=1
      ELSE IF(KDIR .GT. 3) THEN
        IF(ITPIJ .EQ. 4) THEN
          CAN(2:3)='-1'
          CNSIGW=' '
          LONS0=-10
          IDIAGA=2
        ELSE IF(ITPIJ .EQ. 3) THEN
          CAN(3:3)='1'
          CNSIGW='SIGW 1'
          LONS0=NBMIX
          IDIAGA=1
        ENDIF
        ICNL=2
        IDIR=KDIR-3
      ELSE
        CAN(3:3)='1'
        CNSIGW='SIGW 1'
        LONS0=NBMIX
        IDIR=KDIR
        IDIAGA=0
        ICNL=2
      ENDIF
      KRSK=IDIR*NREGIO
      IF (ISADJ .GT. 0) THEN
        IDIREC=-1
      ELSE
        IDIREC=1
      ENDIF
C----
C  CHECK IF VARIATIONAL ACCELERATION ACTIVE
C----
      IF(IACITR .EQ. 0) THEN
        LACVAR=.FALSE.
      ELSE
        LACVAR=.TRUE.
        CALL SETARA(RBASE,NUNKNO,IGAMMA)
        CALL SETARA(RBASE,2*NUNKNO,IRESTE)
      ENDIF
      IF(IREBAL .GT. 0) THEN
        CALL SETARA(RBASE,NGROUP*(NGROUP+1),JREBAL)
        CALL SETARA(RBASE,NREGIO,JPIS)
      ENDIF
C----
C  DETERMINE THERMAL GROUP LIMIT / INITIALIZE IGCONV.
C----
      LFGTHR=.TRUE.
      IF(LONS0 .EQ. -10) THEN
        LMAXIN=NGROUP+1
      ELSE IF(IDIAGA .EQ. 1) THEN
        LMAXIN=1
      ELSE IF(MOD(ITPIJ,2) .NE. 0) THEN
        LMAXIN=IFGUPS(ICNL)
      ELSE
        LMAXIN=1
      ENDIF
      IF(IDIREC .EQ. -1) THEN
C----
C  FOR THERMAL GROUPS TEST IF ADJOINT SOURCES AND ADJOINT FLUXES
C  VANISH IDENTICALLY
C  IF THIS IS THE CASE, 
C    SET THE CONVERGENCE FLAG FOR ALL THESE GROUPS TO 2 
C    SINCE SOLUTION IS REACHED AT FIRST ITERATION
C  OTHERWISE 
c    SET CONVERGENCE FLAG TO 0
C----
        ITHRCV=2
        DO 30 IGR=LMAXIN,NGROUP
          DO 31 IREG=1,NREGIO
            IRLOC=KRSK+KEYFLX(IREG)
            IF(FUNKNO(IRLOC,IGR) .NE. 0.0 .OR.
     >         SUNKNO(IRLOC,IGR) .NE. 0.0      ) THEN
              ITHRCV=0
              GO TO 35
            ENDIF
 31       CONTINUE
 30     CONTINUE
 35     CONTINUE
      ELSE
        ITHRCV=0
      ENDIF
      DO 40 IGR=1,NGROUP
        IF(IGR .LT. LMAXIN) THEN
          IGCONV(IGR)=2
        ELSE
          IGCONV(IGR)=ITHRCV
        ENDIF
  40  CONTINUE
      IF(LMAXIN .EQ. NGROUP+1) MAXINR=1
C----
C COMPUTE FIRST AND LAST GROUP AND INITIALIZE ITHERI
C----
      INEXTG=(1+IDIREC)/2+NGROUP*(1-IDIREC)/2
      ILASTG=NGROUP*(1+IDIREC)/2+LMAXIN*(1-IDIREC)/2
      ITHERI=0
      IF(IPRINT .GT. 1 .AND. MAXINR .GT. 1) THEN
        WRITE(IOUT,6000) CNFDIR(IDIR)
      ENDIF
      WOVER=0.0
      IGMAXD=0
C----
C  OVERALL GROUP ITERATION
C   DIRECT
C    1) FIRST PASS GOES THROUGH ALL GROUPS
C    2) NEXT PASS GOES THROUGH NON CONVERGED THERMAL GROUPS
C   ADJOINT
C    1) FIRST PASSES GOES THROUGH NON CONVERGED THERMAL GROUPS
C    2) LAST PASS GOES THROUGH ALL NON CONVERGED THERMAL
C  50 CONTINUE -> START NEW ITERATION
C  55 CONTINUE -> CONTINUE CURRENT ITERATION
C                 FOR NEXT SET OF GROUPS
C----
      NTOTGS=0
  50  CONTINUE
      IFRSTG=INEXTG
      ITHERI=ITHERI+1
      IF(IDIREC .EQ. 1) THEN
        IFGP=IFRSTG
        ILGP=ILASTG
      ELSE
        ILGP=IFRSTG
        IFGP=ILASTG
      ENDIF
      IF(IPRINT .GE. 5) WRITE(IOUT,6010) ITHERI
  55  CONTINUE
      IALLCV=2
      TRELER=0.0
      IF(IPRINT .GE. 5) WRITE(IOUT,6015) IFRSTG,ILASTG
      DO 110 IGR=IFRSTG,ILASTG,IDIREC
        NTOTGS=NTOTGS+1
        IF(IPRINT .GE. 5) WRITE(IOUT,6012) IGR
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
        CALL LCMSIX(IPSYS,CGRPNM,ILCMUP)
        IF(CNSIGW .EQ. 'DRAGON-S0XSC') THEN
          CALL LCMLEN(IPSYS,CNSIGW,ILCMLN,ILCMTY)
          IF(ILCMLN .EQ. LONS0) THEN
            CALL LCMGET(IPSYS,CNSIGW,SIGS0(0))
          ELSE
            CALL LCMLIB(IPSYS)
            CALL XABORT(NAMSBR//
     >      ': READ ERROR ON LCM RECORD = '//CNSIGW)
          ENDIF
        ELSE
          CALL LCMLEN(IPMACR,CNSIGW,ILCMLN,ILCMTY)
          IF(ILCMLN .EQ. LONS0) THEN
            CALL LCMGET(IPMACR,CNSIGW,SIGS0(1))
          ELSE IF(LONS0 .NE. -10) THEN
            CALL LCMLIB(IPMACR)
            CALL XABORT(NAMSBR//
     >      ': READ ERROR ON LCM RECORD = '//CNSIGW)
          ENDIF
          SIGS0(0)=0.0
        ENDIF
        CALL LCMLEN(IPMACR,'NJJ'//CAN,ILCMLN,ILCMTY)
        IF(ILCMLN .EQ. NBMIX) THEN
          CALL LCMGET(IPMACR,'NJJ'//CAN,NJJ(1))
          CALL LCMGET(IPMACR,'IJJ'//CAN,IJJ(1))
          CALL LCMGET(IPMACR,'IPO'//CAN,IPOS(1))
          CALL LCMGET(IPMACR,'SCA'//CAN,XSCAT(1))
        ELSE IF(LONS0 .NE. -10) THEN
          CALL LCMLIB(IPMACR)
          CALL XABORT(NAMSBR//': FLUX CALCULATION ERROR, '//
     >    'SCATTERING MATRIX SCA'//CAN//' NOT ON LCM')
        ENDIF
C----
C  INITIALISE REGION SOURCES WITH FIXED SOURCES, INCLUDING FISSION
C  SOURCES, AND SAVE FLUX FROM LAST ITERATION
C----
        IF(IDIR .EQ. 0) THEN
          DO 60 IELEM=1,NUNKNO
            SOURC(IELEM)=SUNKNO(IELEM,IGR)
            DELFL(IELEM,IGR)=FUNKNO(IELEM,IGR)
 60       CONTINUE
        ELSE
          DO 61 IELEM=1+KRSK,KRSK+NREGIO
            SOURC(IELEM)=SUNKNO(IELEM,IGR)
            DELFL(IELEM,IGR)=FUNKNO(IELEM,IGR)
 61       CONTINUE
        ENDIF
C----
C  ADD TO REGION SOURCE SCATTERING SOURCES FROM ALL GROUPS
C----
        IF(IDIAGA .EQ. 0) THEN
          DO 70 IREG=1,NREGIO
            IBM=MATCOD(IREG)
            IGSCAT=IJJ(IBM)
            IRLOC=KRSK+KEYFLX(IREG)
            DO 71 JGR=1,NJJ(IBM)
              IF(IGSCAT .NE. IGR) THEN
                 SOURC(IRLOC)=SOURC(IRLOC)
     >           +XSCAT(IPOS(IBM)+JGR-1)*FUNKNO(IRLOC,IGSCAT)
              ELSE IF(IGSCAT .EQ. IGR .AND.
     >                MOD(ITPIJ,2) .EQ. 0) THEN
                 SOURC(IRLOC)=SOURC(IRLOC)
     >           +SIGS0(IBM)*FUNKNO(IRLOC,IGR)
              ENDIF
              IGSCAT=IGSCAT-1
 71         CONTINUE
 70       CONTINUE
        ELSE IF(IDIAGA .EQ. 1) THEN
          DO 80 IREG=1,NREGIO
            IBM=MATCOD(IREG)
            IRLOC=KRSK+KEYFLX(IREG)
            SOURC(IRLOC)=SOURC(IRLOC)
     >         -SIGS0(MATCOD(IREG))*FUNKNO(IRLOC,IGR)/DB2(IGR)
 80       CONTINUE
        ENDIF
C----
C  CALL FMODUL OR FLUDB2 FOR FLUX SOLUTION.
C----
        IF(ILEAK .GE. 5) THEN
          IF(IDIR .GT. 0) THEN
            FACT=DB2(IGR)
            DO 200 IREG=1,NREGIO
              SOURC(KRSK+KEYFLX(IREG))=FACT*SOURC(KRSK+KEYFLX(IREG))
 200        CONTINUE
          ENDIF
          CALL FMODUL(IPSYS ,IPTRK ,IPBTF ,IPRINT,IDIR  ,NREGIO,
     >                NREGIO,MATCOD,VOLUME,KEYFLX,
     >                FUNKNO(KRSK+1,IGR)  ,SOURC(KRSK+1),TITLE )
        ELSE IF(ILEAK .EQ. 0) THEN
          CALL FMODUL(IPSYS ,IPTRK ,IPBTF ,IPRINT,IDIR  ,NREGIO,
     >                NUNKNO,MATCOD,VOLUME,KEYFLX,
     >                FUNKNO(1,IGR)       ,SOURC        ,TITLE )
        ELSE
          CALL FLUDB2(FMODUL,IPSYS ,IPTRK ,IPBTF ,IPRINT,NBMIX ,
     >                NREGIO,NUNKNO,MATCOD,VOLUME,KEYFLX,ILEAK ,
     >                FUNKNO(1,IGR),SOURC ,ISADJ ,TITLE ,
     >                DB2(IGR),SIGT0,SIGS0,PNL(IGR))
        ENDIF
C----
C  COMPUTE DB2 PARAMETER CORRESPONDING TO ACTUAL LEAKAGE
C----
        IF(ILEAK .LT. 5 .AND. IPRINT .GT. 10) THEN
          DB2NEW=FLUFUI(IPSYS,NBMIX,NREGIO,NUNKNO,MATCOD,VOLUME,KEYFLX,
     >                  FUNKNO(1,IGR),SOURC,SIGT0,SIGS0)
          WRITE(IOUT,6014) IGR,DB2(IGR),DB2NEW
        ENDIF
C----
C  COMPUTE MAXIMUM ERROR ON FLUXES.
C----
        GDPHI=0.0
        GMPHI=0.0
        DO 90 IREG=1,NREGIO
          IRLOC=KRSK+KEYFLX(IREG)
          GDPHI=MAX(ABS(FUNKNO(IRLOC,IGR)
     >              -DELFL(IRLOC,IGR)),GDPHI)
          GMPHI=MAX(ABS(FUNKNO(IRLOC,IGR)),GMPHI)
  90    CONTINUE
        IF(IPRINT .GE. 5) THEN
          IF(ILEAK .GE. 5) THEN
            CALL SMODUL(IPSYS ,IPTRK ,IPBTF ,IPRINT,IDIR  ,NREGIO,
     >                  NREGIO,MATCOD,VOLUME,KEYFLX,
     >                  FUNKNO(KRSK+1,IGR)  ,SOURC(KRSK+1),TITLE )
          ELSE
            CALL SMODUL(IPSYS ,IPTRK ,IPBTF ,IPRINT,IDIR  ,NREGIO,
     >                  NUNKNO,MATCOD,VOLUME,KEYFLX,
     >                  FUNKNO(1,IGR)       ,SOURC        ,TITLE )
          ENDIF
          ZL2=0.0D0
          IF(IDIR .EQ. 0) THEN
            DO 100 IELEM=1,NUNKNO
              ZL2=ZL2+SOURC(IELEM)**2
 100        CONTINUE
          ELSE
            DO 101 IELEM=1+KRSK,KRSK+NREGIO
              ZL2=ZL2+SOURC(IELEM)**2
 101        CONTINUE
          ENDIF
          ZL2=SQRT(ZL2)
          WRITE(IOUT,6013) NAMSBR,ZL2
        ENDIF
C----
C   CHECK GROUP CONVERGENCE
C----
        IF(IGCONV(IGR) .EQ. 2) THEN
          IF(IALLCV .EQ. 2) THEN
            INEXTG=IGR+IDIREC
          ENDIF
        ELSE IF(IGCONV(IGR) .EQ. 1) THEN
          IF(GMPHI .EQ. 0.0) THEN
            RELCNV=1.0
          ELSE
            RELCNV=GDPHI/GMPHI
          ENDIF
          IF(TRELER .LT. RELCNV) THEN
            IGMAXD=IGR
            TRELER=RELCNV
          ENDIF
          IF(RELCNV .LT. EPSTHR) THEN
            IGCONV(IGR)=2
            IF(IALLCV .EQ. 2) THEN
              INEXTG=IGR+IDIREC
            ENDIF
          ELSE
            IALLCV=1
          ENDIF
        ELSE IF(IGCONV(IGR) .EQ. 0) THEN
          IF(GMPHI .EQ. 0.0) THEN
            RELCNV=1.0
          ELSE
            RELCNV=GDPHI/GMPHI
          ENDIF
          IF(TRELER .LT. RELCNV) THEN
            IGMAXD=IGR
            TRELER=RELCNV
          ENDIF
          IF(IALLCV .GE. 1) THEN
            IF(RELCNV .LT. EPSTHR) THEN
              IGCONV(IGR)=1
            ELSE
              IALLCV=0
            ENDIF
          ENDIF
        ENDIF
        CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
        CALL LCMSIX(IPSYS,CGRPNM,ILCMDN)
 110  CONTINUE
      IF(IALLCV .EQ. 2) THEN
        IF(IDIREC .EQ. -1 .AND. ILASTG .NE. 1) THEN
          IFGP=1
          IFRSTG=LMAXIN+IDIREC
          ILASTG=1
          GO TO 55
        ENDIF
      ELSE
        ILSG=IFRSTG
        DO 120 IGR=ILASTG,IFRSTG,-IDIREC
          IF(IGCONV(IGR) .NE. 2) THEN
            ILSG=IGR
            GO TO 125
          ENDIF
 120    CONTINUE
 125    CONTINUE
        ILASTG=ILSG
      ENDIF
      IF(IPRINT .GT. 1 .AND. MAXINR .GT. 1) THEN
        WRITE(IOUT,6001) ITHERI,IFGP,ILGP,WOVER,IGMAXD,TRELER
      ENDIF
C----
C  ACCELERATION AND FLUX REBALANCING
C----
      WOVER=0.0
      IF(ITHERI .LT. MAXINR) THEN
        IF(IALLCV .EQ. 0) THEN
C----
C  FLUX REBALANCING.
C----
          IF(IREBAL .GT. 0) THEN
            CALL FLUBAL(IPSYS ,IPMACR,IPRINT,ITPIJ ,KDIR  ,NGROUP,
     >                  NBMIX ,NREGIO,NUNKNO,LEAKSW,ILEAK ,MATCOD,
     >                  VOLUME,KEYFLX,FUNKNO,SUNKNO,ISADJ ,INEXTG,
     >                  ILASTG,DB2   ,SIGT0 ,SIGS0 ,XSCAT ,IJJ   ,
     >                  NJJ   ,IPOS  ,
     >                  RBASE(JREBAL),RBASE(JPIS)  )
          ENDIF
C----
C  PERFORM VARIATIONAL ACCELERATION IN SETS OF
C  (IACITR ACCELERATED ,IFRITR FREE) ITERATIONS
C----
          IF(LACVAR .AND. MOD(ITHERI,IACITR+IFRITR) .GE. IFRITR) THEN
C----
C  PERFORM VARIATIONAL ACCELERATION
C----
            CALL FLUACV(SMODUL,IPSYS ,IPMACR,IPTRK ,IPBTF ,IPRINT,
     >                  ITPIJ, KDIR  ,NGROUP,NBMIX ,NREGIO,NUNKNO,
     >                  ILEAK ,MATCOD,VOLUME,KEYFLX,FUNKNO,SUNKNO,
     >                  ISADJ ,INEXTG,ILASTG,DB2   ,SIGT0 ,SIGS0 ,
     >                  XSCAT ,IJJ   ,NJJ   ,IPOS  ,WOVER ,DELFL ,
     >                  RBASE(IGAMMA),RBASE(IRESTE))
            IF(WOVER .LT. 0.0) THEN
              WOVER=0.0
              LACVAR=.FALSE.
            ENDIF
          ENDIF
          GO TO 50
        ELSE IF(IALLCV .EQ. 1) THEN
          GO TO 50
        ENDIF
      ENDIF
      IF(IALLCV .LT. 2) THEN
        WRITE(IOUT,6002) ITHERI,CNFDIR(IDIR),TRELER,NTOTGS
      ELSE
        IF(IPRINT .GE. 1) THEN
          WRITE(IOUT,6003) ITHERI,CNFDIR(IDIR),TRELER,NTOTGS
        ENDIF
      ENDIF
      EPSTHR=TRELER
      MAXINR=ITHERI
      IF(IREBAL.GT.0) THEN
        CALL RLSARA(RBASE(JPIS))
        CALL RLSARA(RBASE(JREBAL))
      ENDIF
      IF(IACITR.GT.0) THEN
        CALL RLSARA(RBASE(IRESTE))
        CALL RLSARA(RBASE(IGAMMA))
      ENDIF
      RETURN
C----
C  FORMATS:
C----
 6000 FORMAT(/1X,A5/1X,'INNER',8X,'FIRST',8X,'LAST',9X,
     > 'VARIATIONAL ACCEL.',2X,'MOST DIVERGENT     GROUP'/1X,'ITERATION'
     > ,4X,'GROUP',8X,'GROUP',12X,'FACTOR',14X,'GROUP',10X,'ERROR')
 6001 FORMAT(1X,I5,8X,I5,8X,I5,10X,F10.5,9X,I5,10X,1P,E12.5)
 6002 FORMAT(1X,'*** WARNING *** MAXIMUM NUMBER OF INNER ITERATIONS R',
     > 'EACHED (',I5,')'/
     >       1X,'MAXIMUM ERROR ON ',A5,' FOUND           =',1P,E12.5/
     >       1X,'TOTAL NUMBER OF ONE GROUP SOLUTIONS    = ',I12/)
 6003 FORMAT(1X,'CONVERGENCE REACHED AT INNER ITERATION = ',I12/
     >       1X,'MAXIMUM ERROR ON ',A5,' FOUND           = ',1P,E12.5/
     >       1X,'TOTAL NUMBER OF ONE GROUP SOLUTIONS    = ',I12/)
 6010 FORMAT(/1X,'PROCESSING THERMAL ITERATION: ',I10/)
 6012 FORMAT(1X,'CURRENTLY SOLVING IN GROUP: ',I10)
 6013 FORMAT(1X,A6,': L2 NORM OF RESIDUAL VECTOR =',1P,E12.4/)
 6014 FORMAT(1X,' DB2 LEAKAGE PARAMETER FROM DIFFON IN GROUP ',I4,' = ',
     >       1P,E13.4/19X,'ACTUAL DB2 LEAKAGE PARAMETER =',E13.4)
 6015 FORMAT(1X,'FROM GROUP ',I5,2X,'TO GROUP ',I5)
      END
