*DECK XELVOL
      SUBROUTINE XELVOL(   IPRT,   NDIM, NEXTGE,   NCPC,  MINDO,  MAXDO,
     >                   ICORDO,  NSURO,  NVOLO, IDLGEO, INDEXO,
     >                     MAXC, REMESH, MATGEO, VOLSO )
************************************************************************
*                                                                      *
*           NAME: XELVOL                                               *
*      COMPONENT: EXCELL                                               *
*          LEVEL: 4 (CALLED BY 'XELTRK')                               *
*        VERSION: 1.0                                                  *
*       CREATION: 87/01                                                *
*       MODIFIED: 00/03 (R.R.) DECLARE ALL VARIABLE TYPES              *
*         AUTHOR: ROBERT ROY                                           *
*                                                                      *
*     SUBROUTINE: THIS ROUTINE WILL COMPUTE VOLUMES & SURFACES         *
*                                                                      *
*--------+-------------- V A R I A B L E S -------------+--+-----------*
*  NAME  /                  DESCRIPTION                 /IO/MOD(DIMENS)*
*--------+----------------------------------------------+--+-----------*
* IPRT   / INTERMEDIATE PRINTING LEVEL FOR OUTPUT.      /I./INT        *
* NDIM   / # OF DIMENSIONS (2 OR 3).                    /I./INT        *
* NEXTGE / RECTANGULAR(0)/CIRCULAR(1) BOUNDARY.         /I./INT        *
* NCPC   / DIMENSION FOR *MINDO*...                     /I./INT        *
* MINDO  / MIN INDEX VALUES FOR ALL AXES (RECT/CYL).    /I./INT(NCPC)  *
* MAXDO  / MAX INDEX VALUES FOR ALL AXES (RECT/CYL).    /I./INT(NCPC)  *
* ICORDO / PRINCIPAL AXIS DIRECTIONS (X/Y/Z) FOR MESHES./I./INT(NCPC)  *
* NSURO  / # OF SURFACE OF THE GEOMETRY.                /I./INT        *
* NVOLO  / MAX. # OF TRACK SEGMENTS IN A SINGLE TRACK.  /I./INT        *
* IDLGEO / RELATIVE INDEX OF GEOMETRY IN VOLSO.         /I./INT        *
* INDEXO / TO RETRIEVE ZONES IN GEOMETRY.               /I./INT(*)     *
* MAXC   / DIMENSION OF REMESH().                       /I./INT        *
* REMESH / REAL MESHES VALUES (RECT/CYL).               /I./REL(MAXC  )*
* MATGEO / MATERIAL #S USED IN THE GEOMETRY.            /I./INT(*)     *
* VOLSO  / VOLUMES & SURFACES FOR EACH GEOMETRY.        /.O/REL(NGIDL) *
*--------+--------------- R O U T I N E S --------------+--+-----------*
*  NAME  /                  DESCRIPTION                                *
*--------+-------------------------------------------------------------*
* LELCRN / TO DECIDE CROWN/RECTANGLE INTERSECTION.                     *
* XELCRN / TO CALCULATE CYLINDER/RECTANGLE INTERSECTION.               *
************************************************************************
C
      IMPLICIT           NONE
C
      INTEGER            IPRT,NDIM,NEXTGE,NCPC,NSURO,NVOLO,IDLGEO,MAXC
      REAL               VOLSO(*),REMESH(MAXC)
      INTEGER            MINDO(NCPC), MAXDO(NCPC), ICORDO(NCPC),
     >                   INDEXO(4,*), MATGEO(*)
C
      REAL               PI, PIO2
      PARAMETER        ( PI = 3.1415926536, PIO2= 0.5*PI )
      INTEGER            IOUT
      PARAMETER        ( IOUT=6 )
      INTEGER            ICUR(4), IND, I, KSUR, JX, JY, JZ, JCY, JRAY,
     >                   IVO, IXYZ, ICT, ICX, ICY, IVO1, IVO2, IP, IR,
     >                   NESUR, NSUNN, NSURC, NSURM,
     >                   NEVOL, NVONN, NVOLC, NVOLM
      REAL               DELTA(3), XYPOS(2,2), RAYONC(2), CENTEC(2),
     >                   PRODUC, VOLARE, SURF2, DZ, AREAI
      LOGICAL            LELCRN, SWZCYL
      CHARACTER*4        CORIEN(-6:0)
      SAVE               CORIEN
C
      DATA    CORIEN  
     >       / ' Z+ ',' Z- ',' Y+ ',' Y- ',' X+ ',' X- ','    ' /
C
      IND(I)= IDLGEO + I
C
C     VOL & SURF CALCULATION (CARTESIAN MESHES)
      RAYONC(1)= 0.0
      KSUR= MOD(NDIM+1,3)
      DO 50 JX  = MINDO(1)-1, MAXDO(1)
         ICUR(1)= JX
         IF( JX.EQ.MINDO(1)-1 .OR. JX.EQ.MAXDO(1) )THEN
            DELTA(1) = 0.25
         ELSE
            DELTA(1)= REMESH(JX+1)-REMESH(JX)
         ENDIF
      DO 40 JY  = MINDO(2)-1, MAXDO(2)
         ICUR(2)= JY
         IF( JY.EQ.MINDO(2)-1 .OR. JY.EQ.MAXDO(2) )THEN
            DELTA(2) = 0.25
         ELSE
            DELTA(2)= REMESH(JY+1)-REMESH(JY)
         ENDIF
      DO 30 JZ  = MINDO(3)-KSUR, MAXDO(3)+KSUR-1
         ICUR(3)= JZ
         IF( JZ.EQ.MINDO(3)-1 .OR. JZ.EQ.MAXDO(3) )THEN
            DELTA(3) = 0.25
         ELSE
            DELTA(3)= REMESH(JZ+1)-REMESH(JZ)
         ENDIF
         PRODUC= DELTA(1)*DELTA(2)*DELTA(3)
         DO 20 IVO = NSURO, NVOLO
            DO 10 IXYZ= 1, 3
               IF(INDEXO(IXYZ,IND(IVO)).NE.ICUR(IXYZ))GO TO 20
   10       CONTINUE
            VOLSO(IND(IVO))= PRODUC
   20    CONTINUE
   30 CONTINUE
   40 CONTINUE
   50 CONTINUE
      DO 130 JCY= 4, NCPC
         ICT= ICORDO(JCY)
         ICX= MOD(ICT  , 3) + 1
         ICY= MOD(ICT+1, 3) + 1
         CENTEC(1)= REMESH(MINDO(JCY)-2)
         CENTEC(2)= REMESH(MINDO(JCY)-1)
         DO 120 JRAY= MAXDO(JCY), MINDO(JCY), -1
            RAYONC(2)= REMESH(JRAY)
         DO 110 JX  = MINDO(ICX), MAXDO(ICX)-1
            ICUR(ICX)= JX
            XYPOS(1,1)   = REMESH(JX)-CENTEC(1)
            XYPOS(2,1)   = REMESH(JX+1)-CENTEC(1)
         DO 100 JY  = MINDO(ICY), MAXDO(ICY)-1
            ICUR(ICY)= JY
            XYPOS(1,2)   = REMESH(JY)-CENTEC(2)
            XYPOS(2,2)   = REMESH(JY+1)-CENTEC(2)
            IF( .NOT.LELCRN(CENTEC,RAYONC,REMESH(JX),REMESH(JY)) )
     >         GO TO 100
            CALL XELCRN(IPRT,RAYONC(2),1,1,XYPOS,AREAI)
            DO 90 JZ = MINDO(ICT)-KSUR, MAXDO(ICT)+KSUR-1
               ICUR(ICT)= JZ
               IF( JZ.EQ.MINDO(ICT)-1 .OR. JZ.EQ.MAXDO(ICT) )THEN
                  DZ    = 0.25
               ELSE
                  DZ    = REMESH(JZ+1) - REMESH(JZ)
               ENDIF
               VOLARE= AREAI * DZ
               DO 85 IVO1= NSURO, NVOLO
                  ICUR(4)= JRAY-1
                  DO 70 IXYZ= 1, 4
                     IF(INDEXO(IXYZ,IND(IVO1)).NE.ICUR(IXYZ)) GO TO 85
   70             CONTINUE
                  ICUR(4)= JRAY
                  DO 80 IVO2= NSURO, NVOLO
                     DO 75 IXYZ= 1, 4
                      IF(INDEXO(IXYZ,IND(IVO2)).NE.ICUR(IXYZ)) GO TO 80
   75                CONTINUE
                     VOLSO(IND(IVO1))= VOLARE
                     VOLSO(IND(IVO2))= VOLSO(IND(IVO2)) - VOLARE
                     GO TO 85
   80             CONTINUE
   85          CONTINUE
   90       CONTINUE
  100    CONTINUE
  110    CONTINUE
  120    CONTINUE
  130 CONTINUE
      NESUR= 0
      NEVOL= 0
      SWZCYL=.TRUE.
      IF( NEXTGE.EQ.1 )THEN
         DO 700 IVO1= NSURO, NVOLO
           IF( IVO1.LT.0.AND.MATGEO(IND(IVO1)).EQ.0 ) NESUR= NESUR-1
           IF( IVO1.GT.0.AND.MATGEO(IND(IVO1)).LT.0 ) NEVOL= NEVOL+1
 700     CONTINUE
         JRAY = MAXDO(4)
         SURF2= PIO2 * SQRT(REMESH(JRAY))
         ICT= ICORDO(4)
         DO 720 JZ = MINDO(ICT), MAXDO(ICT)-1
            DZ    = REMESH(JZ+1) - REMESH(JZ)
            DO 710 IVO1= 1, NVOLO
              IF( INDEXO(ICT,IND(IVO1)).NE.JZ     ) GO TO 710
              IF( INDEXO( 4,IND(IVO1)).NE.JRAY ) GO TO 710
              SWZCYL= SWZCYL.AND.(MATGEO(IND(IVO1)).LT.0)
              VOLSO(IND(MATGEO(IND(IVO1))))= DZ*SURF2
  710       CONTINUE
  720    CONTINUE
      ENDIF
C
      IF( IPRT.GT.1 )THEN
         NSUNN = NSURO-NESUR-NEVOL
         NSURC = -1
         WRITE(IOUT,'(1H )')
         WRITE(IOUT,'(/35H  CELL SURFACES BEFORE ASSEMBLING         )')
         DO 180 IP  = 1, (9 - NSUNN) / 10
            NSURM= MAX( NSUNN, NSURC-9 )
            WRITE(IOUT,'(10X,10(A5,I6))')
     >           (' SUR ',-IR,IR= NSURC, NSURM, -1)
            WRITE(IOUT,'(8H VALUE  ,2X,1P,10E11.4)')
     >           (4.*VOLSO(IND(IR)),IR=NSURC,NSURM,-1)
            WRITE(IOUT,'(8H ORIENT ,2X,10A11)')
     >           (CORIEN(MATGEO(IND(IR))),IR=NSURC,NSURM,-1)
            WRITE(IOUT,'(1H )')
            NSURC = NSURC - 10
  180    CONTINUE
         NVONN= NVOLO-NEVOL
         NVOLC= 1
         WRITE(IOUT,'(1H )')
         WRITE(IOUT,'( 35H  CELL VOLUMES  BEFORE ASSEMBLING         )')
         DO 190 IP  = 1, (9 + NVONN) / 10
            NVOLM= MIN( NVONN, NVOLC+9 )
            WRITE(IOUT,'(10X,10(A5,I6))')
     >                (' VOL ',IR,IR=NVOLC,NVOLM, 1)
            WRITE(IOUT,'(8H VALUE  ,2X,1P,10E11.4)')
     >               (VOLSO(IND(IR)),IR=NVOLC,NVOLM, 1)
            WRITE(IOUT,'(8H MERGE  ,2X,10I11)')
     >               (MATGEO(IND(IR)),IR=NVOLC,NVOLM, 1)
            WRITE(IOUT,'(1H )')
            NVOLC = NVOLC + 10
  190    CONTINUE
          IF( .NOT.SWZCYL )
     >           CALL XABORT( 'XELVOL: '//
     >                        'CIRCULAR ZONES INCORRECTLY DEFINED' )
      ENDIF
C
      RETURN
      END
